class Bio2Tag:
    """ Class responsible converting tag format annotated text
    to BIO format annotated text 
    
    Attributes:
        text (str): The text that will be converted 
    
    """  

    def __init__(self, text: str='') -> None:
        self.text: str = text
        self.parsed_text: str = ''

    def _prepare_token(self, token_list: list) -> list:        
        """ Private function to handle the BIO formatting
        
        Args: 
            token_list (list): list of BIO token lines
        
        Returns: 
            list: list of ready to print tokens
        
        """

        prepared_token_list = list()
        len_token_list = len(token_list)

        for i, token in enumerate(token_list):
        
            token_split = token.split(' ')
            next_token = ['', ''] if i == len_token_list-1 else token_list[i+1].split(' ')

            if len(token_split) < 2:
                prepared_token_list.append(token_split[0])
            elif token_split[1][0] == 'B' and next_token[1][0] == 'I':
                prepared_token_list.append(f'<{token_split[1][2:]}> {token_split[0]}') 
            elif  token_split[1][0] == 'I' and next_token[1][0] != 'I':
                prepared_token_list.append(f'{token_split[0]} </{token_split[1][2:]}>')
            elif token_split[1][0] == 'B' and next_token[1][0] != 'I':
                prepared_token_list.append(f'<{token_split[1][2:]}> {token_split[0]} </{token_split[1][2:]}>')
            else:
                prepared_token_list.append(token_split[0])

        return prepared_token_list

    def parse(self) -> str:
        """ Function to parse text from BIO format to tag format
        
        Args: 
            None
        
        Returns: 
            str: parsed text in tag format
    
        """
        
        splitted_text = self.text.split('\n')
        self.parsed_text = ' '.join(self._prepare_token(splitted_text))

        return self.parsed_text
    
    def save(self, path: str='./tag_output.txt') -> None:
        """ Function to save the parsed text to a file
        
        Args: 
            path (str): path where the file will be saved
        
        Returns: 
            None

        """
        
        with open(path, 'w') as f:
            f.write(self.parsed_text)


from bs4 import BeautifulSoup


class Tag2Bio:
    """ Class responsible converting tag format annotated text
    to BIO format annotated text 
    
    Attributes:
        text (str): The text that will be converted 
    
    """

    def __init__(self, text: str='') -> None:
        self.text: str = text
        self.soup: BeautifulSoup = BeautifulSoup(
            self.text,
            features='html.parser'
        )
        self.parsed_text: str = ''

    def parse(self) -> str:
        """ Function to parse text from tag format to BIO format
        
        Args: 
            None
        
        Returns: 
            str: parsed text in BIO format

        """
        
        clean_text: str = self.soup.get_text()
        # It's a long story...
        clean_text = clean_text.replace('.', ' .')
        
        clean_text_split: list = clean_text.split()
        aux_structure: list = [['O', '']] * len(clean_text.split())
        tag_infos: list = [
            (''.join(tag.find_all(text=True)), tag.name) 
            for tag in self.soup.find_all()
        ]

        for txt_to_search, tag_name in tag_infos:
            token_qty: int = len(txt_to_search.split())
            for idx in range(len(clean_text_split) - token_qty+1):
                if clean_text_split[idx : idx+token_qty] == txt_to_search.split():
                    aux_structure[idx] = ['B', tag_name]
                    aux_structure[idx+1 : idx+token_qty] = [['I', tag_name]] * (token_qty-1) 
                    break
        
        for token, (bio_tag, tag_name) in zip(clean_text_split, aux_structure):
            separator = '-' if bio_tag != 'O' else ' '
            self.parsed_text += f'{token} {bio_tag}{separator}{tag_name.lower()}\n'

        return self.parsed_text


    def save(self, path: str='./bio_output.txt') -> None:
        """ Function to save the parsed text to a file
        
        Args: 
            path (str): path where the file will be saved
        
        Returns: 
            None
    
        """
        
        with open(path, 'w') as f:
            f.write(self.parsed_text)


from setuptools import setup

setup(name='tag2bio',
    version='0.4',
    description='A package created for translating tag formatted text to BIO formatted text.',
    packages=['tag2bio'],
    install_requires=['beautifulsoup4==4.10.0'],
    zip_safe=False
)